#ifndef Py_ARRAYOBJECT_H
#define Py_ARRAYOBJECT_H
#ifdef __cplusplus
extern "C" {
#endif

#define MAX_ELSIZE 16

enum PyArray_TYPES { PyArray_CHAR, PyArray_UBYTE, PyArray_SBYTE,
			PyArray_SHORT, PyArray_INT, PyArray_LONG,
			PyArray_FLOAT, PyArray_DOUBLE, 
			PyArray_CFLOAT, PyArray_CDOUBLE,
			PyArray_OBJECT,
			PyArray_NTYPES, PyArray_NOTYPE};

typedef void (PyArray_VectorUnaryFunc) Py_FPROTO((char *, int, char *, int, int));

typedef PyObject * (PyArray_GetItemFunc) Py_FPROTO((char *));
typedef int (PyArray_SetItemFunc) Py_FPROTO((PyObject *, char *));

typedef struct {
  PyArray_VectorUnaryFunc *cast[PyArray_NTYPES]; /* Functions to cast to */
					           /* all other types */
  PyArray_GetItemFunc *getitem;
  PyArray_SetItemFunc *setitem;

  int type_num, elsize;
  char *one, *zero;
  char type;

} PyArray_Descr;

#define CONTIGUOUS 1
#define OWN_DIMENSIONS 2
#define OWN_STRIDES 4
#define OWN_DATA 8

typedef struct {
  PyObject_HEAD
  char *data;
  int nd;
  int *dimensions, *strides;
  PyObject *base;
  PyArray_Descr *descr;
  int flags;
} PyArrayObject;

extern DL_IMPORT(PyTypeObject) PyArray_Type;

/* The following are not intended for use in user code, only needed by umath. */
/* If you write your own match library, you might want this function. */
int PyArray_SetNumericOps Py_FPROTO((PyObject *));

extern int PyArray_INCREF(PyArrayObject *ap);

extern int PyArray_XDECREF(PyArrayObject *ap);

/* Export the array error object.  Is this a good idea?  */
PyObject * PyArrayError Py_FPROTO(());

/* Set the array print function to be a python function */
extern void PyArray_SetStringFunction Py_FPROTO((PyObject *op, int repr));

/* Get the PyArray_Descr structure for a typecode */
extern PyArray_Descr *PyArray_DescrFromType Py_FPROTO((int));

/* Cast an array to a different type */
PyObject *PyArray_Cast Py_FPROTO((PyArrayObject *, int));

/* Check the type coercion rules */
extern int PyArray_CanCastSafely Py_FPROTO((int fromtype, int totype));

/* Return the typecode to use for an object if it was an array */
extern int PyArray_ObjectType Py_FPROTO((PyObject *, int));


extern int _PyArray_multiply_list Py_FPROTO((int *lp, int n));

/* The following defines the C API for the array object for most users */

#define PyArray_Check(op) ((op)->ob_type == &PyArray_Type)

#define PyArray_SIZE(mp) (_PyArray_multiply_list((mp)->dimensions, (mp)->nd))
#define PyArray_NBYTES(mp) ((mp)->descr->elsize * PyArray_SIZE(mp))
/* Obviously this needs some work. */
#define PyArray_ISCONTIGUOUS(m) ((m)->flags & CONTIGUOUS)


/* Return the size in number of items of an array */
extern int PyArray_Size Py_FPROTO((PyObject *));


/* Array creation functions */
/* new_array = PyArray_FromDims(n_dimensions, dimensions[n_dimensions], item_type); */
PyObject *PyArray_FromDims Py_PROTO((int, int *, int));

/* array_from_existing_data = PyArray_FromDimsAndData(n_dims, dims[n_dims], item_type, old_data); */
/* WARNING: using PyArray_FromDimsAndData is not reccommended!  It should only be used to refer to */
/* global arrays that will never be freed (like FORTRAN common blocks). */
PyObject *PyArray_FromDimsAndData Py_PROTO((int, int *, int, char *));

/* Initialize from a python object. */

/* PyArray_ContiguousFromObject(object, typecode, min_dimensions, max_dimensions) */
/* if max_dimensions = 0, then any number of dimensions are allowed. */
/* If you want an exact number of dimensions, you should use max_dimensions */
/* = min_dimensions. */

PyObject *PyArray_ContiguousFromObject Py_FPROTO((PyObject *, int, int, int));

/* Same as contiguous, except guarantees a copy of the original data */
PyObject *PyArray_CopyFromObject Py_FPROTO((PyObject *, int, int, int));

/* Shouldn't be used unless you know what you're doing and are not scared by discontiguous arrays */
PyObject *PyArray_FromObject Py_FPROTO((PyObject *, int, int, int));

/* Return either an array, or if passed a 0d array return the appropriate python scalar */
PyObject *PyArray_Return Py_FPROTO((PyArrayObject *));

PyObject *PyArray_Reshape(PyArrayObject *ap, PyObject *shape);
PyObject *PyArray_Copy(PyArrayObject *ap);
PyObject *PyArray_Take(PyObject *ap, PyObject *items, int axis);

/*Getting arrays in various useful forms. */
extern int PyArray_As1D(PyObject **op, char **ptr, int *d1, int typecode);
extern int PyArray_As2D(PyObject **op, char ***ptr, int *d1, int *d2, int
			typecode);
extern int PyArray_Free(PyObject *op, char *ptr);

#ifdef __cplusplus
}
#endif
#endif /* !Py_ARRAYOBJECT_H */


